@@ -641,9 +641,20 @@ class Client(abc.ABC):
         Returns:
             :obj:`None`
         """
-        for mapping in self._command_invocation_mapping.values():
-            for collection in mapping.values():
-                collection.remove(command)
+        # If this is a group, we need to manually remove all subcommands from the command invocation mapping
+        # since the collection.remove() only handles the top-level group
+        if isinstance(command, groups.Group):
+            for guild_id, mapping in self._command_invocation_mapping.items():
+                self._command_invocation_mapping[guild_id] = {
+                    path: collection
+                    for path, collection in mapping.items()
+                    if not (len(path) > 1 and path[0] == command.name)
+                }
+        else:
+            # For regular commands, just remove them from collections
+            for mapping in self._command_invocation_mapping.values():
+                for collection in mapping.values():
+                    collection.remove(command)
 
         self._registered_commands.pop(command, None)
 
@@ -716,6 +727,10 @@ class Client(abc.ABC):
             self._extensions.add(path)
             LOGGER.info("extension %r loaded successfully", path)
 
+        # ensure that any commands are made available for the interaction handler
+        if self._started:
+            await self.sync_application_commands(_force_no_api_call=True)
+
     async def load_extensions_from_package(self, package: types.ModuleType, *, recursive: bool = False) -> None:
         """
         Load all extension modules from the given package. Ignores any files with a name that starts with an underscore.
@@ -866,7 +881,7 @@ class Client(abc.ABC):
         self._application = await self.rest.fetch_application()
         return self._application
 
-    async def sync_application_commands(self) -> None:
+    async def sync_application_commands(self, *, _force_no_api_call: bool = False) -> None:
         """
         Sync all application commands registered to the bot with discord. Also, properly registers any commands
         with localization enabled for the command name as well as any commands using deferred registration.
@@ -916,6 +931,9 @@ class Client(abc.ABC):
                 for command_path, actual_command in all_commands.items():
                     self._command_invocation_mapping[snowflake][command_path].put(actual_command)
 
+        if _force_no_api_call:
+            return
+
         if self.sync_commands:
             await sync.sync_application_commands(self)
 
