@@ -54,11 +54,15 @@ if t.TYPE_CHECKING:
     from lightbulb import commands
     from lightbulb import context
     from lightbulb import localization
+    from lightbulb.internal import types
 
     AutocompleteProvider = Callable[[context.AutocompleteContext[context.T]], Awaitable[t.Any]]
 
 T = t.TypeVar("T")
-D = t.TypeVar("D")
+
+DefaultT = t.TypeVar("DefaultT")
+ConvertedT = t.TypeVar("ConvertedT")
+
 CtxMenuOptionReturn: t.TypeAlias = t.Union[hikari.User, hikari.Message]
 
 
@@ -73,7 +77,7 @@ class Choice(t.Generic[T]):
 
 
 @dataclasses.dataclass(slots=True)
-class OptionData(t.Generic[D]):
+class OptionData(t.Generic[DefaultT, ConvertedT]):
     """
     Dataclass for storing information about an option necessary for command creation.
 
@@ -90,7 +94,7 @@ class OptionData(t.Generic[D]):
     localize: bool = False
     """Whether the name and description of the option should be interpreted as localization keys."""
 
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED
     """The default value for the option."""
     choices: hikari.UndefinedOr[Sequence[Choice[t.Any]]] = hikari.UNDEFINED
     """The choices for the option."""
@@ -110,7 +114,14 @@ class OptionData(t.Generic[D]):
     autocomplete: bool = False
     """Whether autocomplete is enabled for the option."""
     autocomplete_provider: hikari.UndefinedOr[AutocompleteProvider[t.Any]] = hikari.UNDEFINED
-    """The provider to use to resolve autocomplete interactions for this command."""
+    """The provider to use to resolve autocomplete interactions for this option."""
+
+    converter: t.Callable[[context.Context, t.Any], ConvertedT] | None = None
+    """
+    The converter to use for this option.
+
+    .. versionadded:: 3.1.0
+    """
 
     _localized_name: str = dataclasses.field(init=False, default="")
     _localized_description: str = dataclasses.field(init=False, default="")
@@ -199,7 +210,7 @@ class OptionData(t.Generic[D]):
         )
 
 
-class Option(t.Generic[T, D]):
+class Option(t.Generic[DefaultT, ConvertedT]):
     """
     Descriptor class representing a command option.
 
@@ -225,15 +236,24 @@ class Option(t.Generic[T, D]):
 
     __slots__ = ("_data", "_unbound_default")
 
-    def __init__(self, data: OptionData[D], default_when_not_bound: T) -> None:
+    def __init__(
+        self,
+        data: OptionData[DefaultT, ConvertedT],
+        default_when_not_bound: DefaultT,
+    ) -> None:
         self._data = data
         self._unbound_default = default_when_not_bound
 
-    def __get__(self, instance: commands.CommandBase | None, owner: type[commands.CommandBase]) -> T | D:
+    def __get__(
+        self, instance: commands.CommandBase | None, owner: type[commands.CommandBase]
+    ) -> DefaultT | ConvertedT:
         if instance is None or getattr(instance, "_current_context", None) is None:
             return self._unbound_default
 
-        return instance._resolve_option(self)
+        if self._data._localized_name not in instance._resolved_option_cache:
+            raise RuntimeError(f"Tried to access option {self._data._localized_name} before resolving options.")
+
+        return t.cast("DefaultT", instance._resolved_option_cache[self._data._localized_name])
 
 
 class ContextMenuOption(Option[CtxMenuOptionReturn, CtxMenuOptionReturn]):
@@ -291,24 +311,57 @@ class ContextMenuOption(Option[CtxMenuOptionReturn, CtxMenuOptionReturn]):
         return message
 
 
+@t.overload
 def string(
     name: str,
     description: str,
     /,
     *,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
     choices: hikari.UndefinedOr[Sequence[Choice[str]]] = hikari.UNDEFINED,
     min_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
     max_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
     autocomplete: hikari.UndefinedOr[AutocompleteProvider[str]] = hikari.UNDEFINED,
-) -> str | D:
+) -> str | DefaultT: ...
+
+
+@t.overload
+def string(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, str], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[str]]] = hikari.UNDEFINED,
+    min_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    max_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[str]] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
+def string(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, str], types.MaybeAwaitable[ConvertedT]] | None = None,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[str]]] = hikari.UNDEFINED,
+    min_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    max_length: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[str]] = hikari.UNDEFINED,
+) -> str | DefaultT | ConvertedT:
     """
     A string option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -320,25 +373,58 @@ def string(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
-    return t.cast(
-        "str",
-        Option(
-            OptionData(
-                type=hikari.OptionType.STRING,
-                name=name,
-                description=description,
-                localize=localize,
-                default=default,
-                choices=choices,
-                min_length=min_length,
-                max_length=max_length,
-                autocomplete=autocomplete is not hikari.UNDEFINED,
-                autocomplete_provider=autocomplete,
-            ),
-            utils.EMPTY,
+    opt = Option(
+        OptionData(
+            type=hikari.OptionType.STRING,
+            name=name,
+            description=description,
+            localize=localize,
+            default=default,
+            choices=choices,
+            min_length=min_length,
+            max_length=max_length,
+            autocomplete=autocomplete is not hikari.UNDEFINED,
+            autocomplete_provider=autocomplete,
+            converter=converter,
         ),
+        utils.EMPTY,
     )
+    return t.cast("str", opt)
+
+
+@t.overload
+def integer(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[int]]] = hikari.UNDEFINED,
+    min_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    max_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[int]] = hikari.UNDEFINED,
+) -> int | DefaultT: ...
+
+
+@t.overload
+def integer(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, int], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[int]]] = hikari.UNDEFINED,
+    min_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    max_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[int]] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
 
 
 def integer(
@@ -346,19 +432,21 @@ def integer(
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, int], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
     choices: hikari.UndefinedOr[Sequence[Choice[int]]] = hikari.UNDEFINED,
     min_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
     max_value: hikari.UndefinedOr[int] = hikari.UNDEFINED,
     autocomplete: hikari.UndefinedOr[AutocompleteProvider[int]] = hikari.UNDEFINED,
-) -> int | D:
+) -> int | DefaultT | ConvertedT:
     """
     An integer option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -370,6 +458,9 @@ def integer(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "int",
@@ -385,26 +476,52 @@ def integer(
                 max_value=max_value,
                 autocomplete=autocomplete is not hikari.UNDEFINED,
                 autocomplete_provider=autocomplete,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def boolean(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> bool | DefaultT: ...
+
+
+@t.overload
+def boolean(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, bool], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def boolean(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, bool], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
-) -> bool | D:
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> bool | DefaultT | ConvertedT:
     """
     A boolean option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -412,6 +529,9 @@ def boolean(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "bool",
@@ -422,30 +542,64 @@ def boolean(
                 description=description,
                 localize=localize,
                 default=default,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def number(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[float]]] = hikari.UNDEFINED,
+    min_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
+    max_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[float]] = hikari.UNDEFINED,
+) -> float | DefaultT: ...
+
+
+@t.overload
+def number(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, float], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    choices: hikari.UndefinedOr[Sequence[Choice[float]]] = hikari.UNDEFINED,
+    min_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
+    max_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
+    autocomplete: hikari.UndefinedOr[AutocompleteProvider[float]] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def number(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, float], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
     choices: hikari.UndefinedOr[Sequence[Choice[float]]] = hikari.UNDEFINED,
     min_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
     max_value: hikari.UndefinedOr[float] = hikari.UNDEFINED,
     autocomplete: hikari.UndefinedOr[AutocompleteProvider[float]] = hikari.UNDEFINED,
-) -> float | D:
+) -> float | DefaultT | ConvertedT:
     """
     A numeric (float) option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -457,6 +611,9 @@ def number(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "float",
@@ -472,26 +629,52 @@ def number(
                 max_value=max_value,
                 autocomplete=autocomplete is not hikari.UNDEFINED,
                 autocomplete_provider=autocomplete,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def user(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.User | DefaultT: ...
+
+
+@t.overload
+def user(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, hikari.User], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def user(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, hikari.User], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
-) -> hikari.User | D:
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.User | DefaultT | ConvertedT:
     """
     A user option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -499,6 +682,9 @@ def user(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "hikari.User",
@@ -509,27 +695,55 @@ def user(
                 description=description,
                 localize=localize,
                 default=default,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def channel(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    channel_types: hikari.UndefinedOr[Sequence[hikari.ChannelType]] = hikari.UNDEFINED,
+) -> hikari.PartialChannel | DefaultT: ...
+
+
+@t.overload
+def channel(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, hikari.PartialChannel], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+    channel_types: hikari.UndefinedOr[Sequence[hikari.ChannelType]] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def channel(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, hikari.PartialChannel], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
     channel_types: hikari.UndefinedOr[Sequence[hikari.ChannelType]] = hikari.UNDEFINED,
-) -> hikari.PartialChannel | D:
+) -> hikari.PartialChannel | DefaultT | ConvertedT:
     """
     A channel option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -538,6 +752,9 @@ def channel(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "hikari.PartialChannel",
@@ -549,26 +766,52 @@ def channel(
                 localize=localize,
                 default=default,
                 channel_types=channel_types,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def role(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Role | DefaultT: ...
+
+
+@t.overload
+def role(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, hikari.Role], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def role(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, hikari.Role], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
-) -> hikari.Role | D:
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Role | DefaultT | ConvertedT:
     """
     A role option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -576,6 +819,9 @@ def role(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "hikari.Role",
@@ -586,26 +832,52 @@ def role(
                 description=description,
                 localize=localize,
                 default=default,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def mentionable(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Snowflake | DefaultT: ...
+
+
+@t.overload
+def mentionable(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, hikari.Snowflake], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def mentionable(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, hikari.Snowflake], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
-) -> hikari.Snowflake | D:
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Snowflake | DefaultT | ConvertedT:
     """
     A mentionable (snowflake) option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -613,6 +885,9 @@ def mentionable(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "hikari.Snowflake",
@@ -623,26 +898,52 @@ def mentionable(
                 description=description,
                 localize=localize,
                 default=default,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
     )
 
 
+@t.overload
+def attachment(
+    name: str,
+    description: str,
+    /,
+    *,
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Attachment | DefaultT: ...
+
+
+@t.overload
+def attachment(
+    name: str,
+    description: str,
+    /,
+    *,
+    converter: t.Callable[[context.Context, hikari.Attachment], types.MaybeAwaitable[ConvertedT]],
+    localize: bool = False,
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> DefaultT | ConvertedT: ...
+
+
 def attachment(
     name: str,
     description: str,
     /,
     *,
+    converter: t.Callable[[context.Context, hikari.Attachment], types.MaybeAwaitable[ConvertedT]] | None = None,
     localize: bool = False,
-    default: hikari.UndefinedOr[D] = hikari.UNDEFINED,
-) -> hikari.Attachment | D:
+    default: hikari.UndefinedOr[DefaultT] = hikari.UNDEFINED,
+) -> hikari.Attachment | DefaultT | ConvertedT:
     """
     An attachment option.
 
     Args:
         name: The name of the option.
         description: The description of the option.
+        converter: The converter to be used to convert the value to a custom type.
         localize: Whether to localize this option's name and description. If :obj:`True`, then the
             ``name`` and ``description`` arguments will instead be interpreted as localization keys from which the
             actual name and description will be retrieved. Defaults to :obj:`False`.
@@ -650,6 +951,9 @@ def attachment(
 
     Returns:
         Descriptor allowing access to the option value from within a command invocation.
+
+    .. versionadded:: 3.1.0
+        The ``converter`` argument.
     """
     return t.cast(
         "hikari.Attachment",
@@ -660,6 +964,7 @@ def attachment(
                 description=description,
                 localize=localize,
                 default=default,
+                converter=converter,
             ),
             utils.EMPTY,
         ),
